#---------------------------------------------------------------------------------
# Build scripts for devkitARM/devkitPPC release 8
#---------------------------------------------------------------------------------


#---------------------------------------------------------------------------------
# specify some urls to download the source packages from
#---------------------------------------------------------------------------------

BINUTILS_URL="http://ftp.gnu.org/gnu/binutils/binutils-2.15.tar.bz2"

GCC_CORE_URL="ftp://ftp.gwdg.de/pub/misc/gcc/releases/gcc-3.4.1/gcc-core-3.4.1.tar.bz2"
GCC_GPP_URL="ftp://ftp.gwdg.de/pub/misc/gcc/releases/gcc-3.4.1/gcc-g++-3.4.1.tar.bz2"

NEWLIB_URL="ftp://sources.redhat.com/pub/newlib/newlib-1.12.0.tar.gz"

BINUTILS="binutils-2.15.tar.bz2"

GCC_CORE="gcc-core-3.4.1.tar.bz2"
GCC_GPP="gcc-g++-3.4.1.tar.bz2"

NEWLIB="newlib-1.12.0.tar.gz"


#---------------------------------------------------------------------------------
# Ask whether to download the source packages or not
#---------------------------------------------------------------------------------

VERSION=0

while [ $VERSION -eq 0 ]
do
  echo
  echo "This script will build and install your devkit. Please select the one you require"
  echo
  echo "1: build devkitARM (gba gp32)"
  echo "2: build devkitPPC (gamecube)"
  read VERSION

  if [ $VERSION -ne 1 ] && [ $VERSION -ne 2 ]
  then
      VERSION=0
  fi
done

if [ $VERSION -eq 1 ]
then
  scriptdir='./dka-scripts'
  package='devkitARM'
  builddir=arm-elf
else
  scriptdir='./dkp-scripts'
  package='devkitPPC'
  builddir=powerpc-elf
fi

DOWNLOAD=0

while [ $DOWNLOAD -eq 0 ]
do
  echo
  echo "The installation requires binutils 2.15, gcc 3.3.3 and newlib 1.12.0.  Please select an option:"
  echo
  echo "1: I have already downloaded the source packages"
  echo "2: Download the packages for me (requires wget)"
  read DOWNLOAD

  if [ $DOWNLOAD -ne 1 ] && [ $DOWNLOAD -ne 2 ]
  then
      DOWNLOAD=0
  fi
done


#---------------------------------------------------------------------------------
# Get preferred installation directory and set paths to the sources
#---------------------------------------------------------------------------------
echo
echo "Please enter the directory where you would like '$package' to be installed:"
echo "for mingw/msys you must use <drive>:/<install path> or you will have include path problems"
read INSTALLDIR
echo

if [ $DOWNLOAD -eq 1 ]
then
    FOUND=0
    while [ $FOUND -eq 0 ]
    do
      echo
      echo "Please enter the full path to the directory that contains the source packages:"
      read SRCDIR

      if [ ! -f $SRCDIR/$BINUTILS ]
      then
	  echo "Error: $BINUTILS not found in $SRCDIR"
	  exit
      else
	  FOUND=1
      fi

      if [ ! -f $SRCDIR/$GCC_CORE ]
      then
	  echo "Error: $GCC_CORE not found in $SRCDIR"
	  exit
      else
	  FOUND=1
      fi

      if [ ! -f $SRCDIR/$GCC_GPP ]
      then
	  echo "Error: $GCC_GPP not found in $SRCDIR"
	  exit
      else
	  FOUND=1
      fi

      if [ ! -f $SRCDIR/$NEWLIB ]
      then
	  echo "Error: $NEWLIB not found in $SRCDIR"
	  exit
      else
	  FOUND=1
      fi
    done

else

    wget -c $BINUTILS_URL
    if [ ! -f $BINUTILS ]
    then
	echo "Error: Failed to download "$BINUTILS
	exit
    fi

    wget -c $GCC_CORE_URL
    if [ ! -f $GCC_CORE ]
    then
	echo "Error: Failed to download "$GCC_CORE
	exit
    fi

    wget -c $GCC_GPP_URL
    if [ ! -f $GCC_GPP ]
    then
	echo "Error: Failed to download "$GCC_GPP
	exit
    fi

    wget -c $NEWLIB_URL
    if [ ! -f newlib-1.12.0.tar.gz ]
    then
	echo "Error: Failed to download "$NEWLIB
	exit
    fi

    SRCDIR=`pwd`
fi

BINUTILS_SRCDIR="binutils-2.15/"
GCC_SRCDIR="gcc-3.4.1/"
NEWLIB_SRCDIR="newlib-1.12.0/"

echo
echo 'Ready to install '$package' in '$INSTALLDIR
echo
echo 'press return to continue'

read dummy


#---------------------------------------------------------------------------------
# Extract source packages
#---------------------------------------------------------------------------------

BUILDSCRIPTDIR=`pwd`

echo "Extracting $BINUTILS"
tar xjvf $SRCDIR/$BINUTILS


echo "Extracting $GCC_CORE"
tar xjvf $SRCDIR/$GCC_CORE

echo "Extracting $GCC_GPP"
tar xjvf $SRCDIR/$GCC_GPP

echo "Extracting $NEWLIB"
tar xzvf $SRCDIR/$NEWLIB


#---------------------------------------------------------------------------------
# apply patches
#---------------------------------------------------------------------------------
patch -p1 -b -i `pwd`/patches/devkit-binutils-2.15.patch -d $BINUTILS_SRCDIR
patch -p1 -b -i `pwd`/patches/devkit-gcc-3.4.1.patch -d $GCC_SRCDIR



#---------------------------------------------------------------------------------
# compilation options
#---------------------------------------------------------------------------------


#---------------------------------------------------------------------------------
# only necessary when Darwin gcc is 3.1 or earlier, to add a check for this here
#---------------------------------------------------------------------------------
if test $(uname -s | grep Darwin)
then
  export CFLAGS = "-O2 -pipe -no-cpp-precomp -DHAVE_DESIGNATED_INITIALIZERS=0"
  export LDFLAGS=''

else

  export CFLAGS='-O2 -pipe'
  export LDFLAGS='-s'

fi

export CXXFLAGS='-O2 -pipe'
export DEBUG_FLAGS=''


#---------------------------------------------------------------------------------
# Build and install devkit components
#---------------------------------------------------------------------------------

$scriptdir/build-binutils.sh ../../$BINUTILS_SRCDIR $INSTALLDIR

#---------------------------------------------------------------------------------
# Add installed binutils to the path
#---------------------------------------------------------------------------------
export PATH=$PATH:$(echo $INSTALLDIR | sed -e 's/^\([a-zA-Z]\):/\/\1/')/bin

$scriptdir/build-gcc.sh ../../$GCC_SRCDIR $INSTALLDIR
$scriptdir/build-newlib.sh ../../$NEWLIB_SRCDIR $INSTALLDIR
$scriptdir/build-final.sh $INSTALLDIR

$scriptdir/build-crtls.sh $INSTALLDIR
$scriptdir/build-tools.sh $INSTALLDIR


#---------------------------------------------------------------------------------
# Clean up temporary files and source directories
#---------------------------------------------------------------------------------

echo
echo "Would you like to delete temporary files? [Y/n]"
read answer

if [ $answer == "y" ] || [ $answer == "Y" ]
then
    echo "rm -fr "$builddir
    rm -fr $builddir
fi

echo
echo "Would you like to delete the source directories? [Y/n]"
read answer

if [ $answer == "y" ] || [ $answer == "Y" ]
then
    echo "rm -fr $BINUTILS_SRCDIR $GCC_SRCDIR $NEWLIB_SRCDIR "
    rm -fr $BINUTILS_SRCDIR $GCC_SRCDIR $NEWLIB_SRCDIR
fi

echo
echo "Would you like to delete the source packages? [Y/n]"
read answer

if [ $answer == "y" ] || [ $answer == "Y" ]
then
    echo "rm $BINUTILS $GCC_CORE $GCC_GPP $NEWLIB -Rf"
    cd $SRCDIR
    rm -fr $BINUTILS $GCC_CORE $GCC_GPP $NEWLIB
fi

echo
echo "note: Add the following to your PATH variable;  $INSTALLDIR/bin"
echo
