#ifndef __NETWORK_H__
#define __NETWORK_H__

#include <gctypes.h>
#include <sys/time.h>
#include <sys/types.h>

#define INVALID_SOCKET	(~0)
#define SOCKET_ERROR	(-1)

#define SOCK_STREAM     1
#define SOCK_DGRAM      2
#define SOCK_RAW        3

#define AF_UNSPEC       0
#define AF_INET         2
#define PF_INET         AF_INET
#define PF_UNSPEC       AF_UNSPEC

#define IPPROTO_IP      0
#define IPPROTO_TCP     6
#define IPPROTO_UDP     17

#define INADDR_ANY      0
#define INADDR_BROADCAST 0xffffffff

/* Flags we can use with send and recv. */
#define MSG_DONTWAIT    0x40            /* Nonblocking i/o for this operation only */

#ifndef O_NONBLOCK
#define O_NONBLOCK    04000U
#endif

#ifndef FD_SET
  #undef  FD_SETSIZE
  #define FD_SETSIZE    16
  #define FD_SET(n, p)  ((p)->fd_bits[(n)/8] |=  (1 << ((n) & 7)))
  #define FD_CLR(n, p)  ((p)->fd_bits[(n)/8] &= ~(1 << ((n) & 7)))
  #define FD_ISSET(n,p) ((p)->fd_bits[(n)/8] &   (1 << ((n) & 7)))
  #define FD_ZERO(p)    memset((void*)(p),0,sizeof(*(p)))

  typedef struct fd_set {
	u8 fd_bits [(FD_SETSIZE+7)/8];
  } fd_set;

  struct timeval {
    s32    tv_sec;         /* seconds */
    s32    tv_usec;        /* and microseconds */
  };

#endif

#ifndef socklen_t
#define socklen_t u32
#endif

#ifndef htons
#define htons(x) (x)
#endif
#ifndef ntohs
#define ntohs(x) (x)
#endif
#ifndef htonl
#define htonl(x) (x)
#endif
#ifndef ntohl
#define ntohl(x) (x)
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef HAVE_IN_ADDR
struct in_addr {
  u32 s_addr;
};
#endif

struct sockaddr_in {
  u8 sin_len;
  u8 sin_family;
  u16 sin_port;
  struct in_addr sin_addr;
  s8 sin_zero[8];
};

struct sockaddr {
  u8 sa_len;
  u8 sa_family;
  s8 sa_data[14];
};

u32 if_config(const char *pszIP,const char *pszGW,const char *pszMASK,boolean use_dhcp);
u32 inet_addr(const char *cp);

u32 net_init();
u32 net_socket(u32 domain,u32 type,u32 protocol);
u32 net_bind(u32 s,struct sockaddr *name,socklen_t namelen);
u32 net_listen(u32 s,u32 backlog);
u32 net_accept(u32 s,struct sockaddr *addr,socklen_t *addrlen);
u32 net_connect(u32 s,struct sockaddr *,socklen_t);
u32 net_write(u32 s,void *data,u32 size);
u32 net_send(u32 s,void *data,u32 size,u32 flags);
u32 net_sendto(u32 s,void *data,u32 len,u32 flags,struct sockaddr *to,socklen_t tolen);
u32 net_recv(u32 s,void *mem,u32 len,u32 flags);
u32 net_recvfrom(u32 s,void *mem,u32 len,u32 flags,struct sockaddr *from,socklen_t *fromlen);
u32 net_read(u32 s,void *mem,u32 len);
u32 net_close(u32 s);
u32 net_select(u32 s,fd_set *readset,fd_set *writeset,fd_set *exceptset,struct timeval *timeout);

#ifdef __cplusplus
	}
#endif

#endif
